from cogito.core.config.file import build_config_file
from cogito.core.utils import (
    create_request_model,
    get_predictor_handler_return_type,
    instance_class,
    wrap_handler,
)
from cogito.core.exceptions import NoSetupMethodError


# SDK Predictor class
class Predict:

    # Initialize predictor
    def __init__(self, config_path):
        # Build config
        config = build_config_file(config_path)

        # Get predictor path and instance
        self.predictor_path = config.cogito.get_predictor
        self.predictor_instance = instance_class(config.cogito.get_predictor)

        # Create input model from payload
        _, self.input_model_class = create_request_model(
            self.predictor_path, self.predictor_instance.predict
        )

        # Get response model type
        self.response_model = get_predictor_handler_return_type(self.predictor_instance)

    # Setup predictor calling setup method in the user's code
    def setup(self):
        try:
            if hasattr(self.predictor_instance, "setup") and callable(
                getattr(self.predictor_instance, "setup")
            ):
                self.predictor_instance.setup()
            else:
                raise NoSetupMethodError(self.predictor_instance.__class__.__name__)
        except Exception as e:
            raise Exception(f"Error setting up the predictor: {e}")

    # Run predictor using the input model in the user's code
    def run(self, payload_data: dict) -> dict:
        input_model = self.input_model_class(**payload_data)

        # Wrap handler with response model
        handler = wrap_handler(
            descriptor=self.predictor_path,
            original_handler=self.predictor_instance.predict,
            response_model=self.response_model,
        )

        # Call handler with input model
        response = handler(input_model)

        # Print response in JSON format
        return response



import json

import click

from cogito.core.exceptions import ConfigFileNotFoundError, NoSetupMethodError
from cogito.lib.training import Trainer


@click.command()
@click.option("--payload", type=str, required=True, help="The payload for the training")
@click.pass_obj
def train(ctx: click.Context, payload: str) -> None:
    """
    Run a cogito training with the specified payload, printing the result to stdout.

    Example: python -m cogito.cli train --payload '{"key": "value"}'
    """

    # Load config and payload
    try:
        config_path = ctx.get("config_path")
        payload_data = json.loads(payload)
    except ConfigFileNotFoundError as e:
        click.echo(f"Config file not found: {e}", err=True, color=True)
        exit(1)
    except Exception as e:
        click.echo(f"Error: {e}", err=True, color=True)
        exit(1)

    # Initialize trainer
    try:
        trainer = Trainer(config_path)
    except Exception as e:
        click.echo(f"Error: {e}", err=True, color=True)
        exit(1)

    # Setup trainer
    try:
        trainer.setup()
    except NoSetupMethodError as e:
        click.echo(f"Warning: {e}", err=True, color=True)
    except Exception as e:
        click.echo(f"Error: {e}", err=True, color=True)
        exit(1)

    # Run trainer
    try:
        result = trainer.run(payload_data, run_setup=True)
        click.echo(result)
    except Exception as e:
        # print stack trace
        # traceback.print_exc()
        click.echo(f"Error: {e}", err=True, color=True)
        exit(1)



import json

import click

from cogito.core.exceptions import ConfigFileNotFoundError, NoSetupMethodError
from cogito.lib.prediction import Predict


@click.command()
@click.option(
    "--payload", type=str, required=True, help="The payload for the prediction"
)
@click.pass_obj
def predict(ctx: click.Context, payload: str) -> None:
    """
    Run a cogito prediction with the specified payload, printing the result to stdout.

    Example: python -m cogito.cli predict --payload '{"key": "value"}'
    """

    # Load config and initialize predictor
    try:
        config_path = ctx.get("config_path")
        payload_data = json.loads(payload)
        predictor = Predict(config_path)
    except Exception as e:
        click.echo(f"Error initializing the predictor: {e}", err=True, color=True)
        exit(1)

    # Setup predictor
    try:
        predictor.setup()
    except NoSetupMethodError as e:
        click.echo(f"Warning: {e}", err=True, color=True)
    except Exception as e:
        click.echo(f"Error setting up the predictor: {e}", err=True, color=True)
        exit(1)

    # Run predictor
    try:
        result = predictor.run(payload_data)

        click.echo(result.model_dump_json(indent=4))
    except ConfigFileNotFoundError as e:
        click.echo(f"Config file not found: {e}", err=True, color=True)
        exit(1)
    except Exception as e:
        # print stack trace
        # traceback.print_exc()
        click.echo(f"Error: {e}", err=True, color=True)
        exit(1)



from cogito.core.config.file import build_config_file
from cogito.core.utils import instance_class
from cogito.core.exceptions import NoSetupMethodError


# SDK Trainer class
class Trainer:

    # Initialize trainer
    def __init__(self, config_path):
        # Build config
        config = build_config_file(config_path)

        # Get trainer path and instance
        self.config_path = config_path
        self.trainer = instance_class(config.cogito.get_trainer)

    # Setup trainer calling setup method in the user's code
    def setup(self):
        # Run setup if needed
        try:
            if hasattr(self.trainer, "setup") and callable(
                getattr(self.trainer, "setup")
            ):
                self.trainer.setup()
            else:
                raise NoSetupMethodError(self.trainer.__class__.__name__)
        except Exception as e:
            raise Exception(f"Error setting up the trainer: {e}")

    # Run training calling train method in the user's code
    def run(self, payload_data, run_setup=True):
        # Call train method with payload data
        try:
            result = self.trainer.train(**payload_data)
        except Exception as e:
            raise Exception(f"Error training the model: {e}")

        return result
